package br.com.xmrig

import android.content.*
import android.os.FileObserver
import android.os.IBinder
import android.os.RemoteException
import android.util.Log
import com.facebook.react.bridge.*
import com.facebook.react.modules.core.DeviceEventManagerModule.RCTDeviceEventEmitter
import kotlinx.serialization.decodeFromString
import kotlinx.serialization.json.Json
import org.greenrobot.eventbus.EventBus
import org.greenrobot.eventbus.Subscribe
import org.greenrobot.eventbus.ThreadMode
import java.lang.Exception
import com.facebook.react.bridge.ReactMethod
import java.io.File
import java.util.*
import android.os.BatteryManager

import android.content.Context.BATTERY_SERVICE
import br.com.xmrig.events.*
import br.com.xmrig.services.IXMRigAPIService
import br.com.xmrig.services.ThermalService
import br.com.xmrig.services.XMRigAPIService
import br.com.xmrig.utils.CPUTemperatureHelper
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch
import kotlinx.coroutines.newSingleThreadContext
import kotlinx.coroutines.runBlocking


class RNReactXMRigModule(context: ReactApplicationContext) : ReactContextBaseJavaModule(context) {

    var miningService:IMiningService? = null
    var xmrigAPIService: IXMRigAPIService? = null
    var isMining = false

    private val serverConnection = object: ServiceConnection {
       
        override fun onServiceConnected(className: ComponentName?, service: IBinder?) {
            if (className != null) {
                Log.d("className", className.className)
            };
            when(className?.className) {
                "br.com.xmrig.MiningService" -> {
                    miningService = IMiningService.Stub.asInterface(service)
                }
                "br.com.xmrig.services.XMRigAPIService" -> {
                    xmrigAPIService = IXMRigAPIService.Stub.asInterface(service)
                }
            }
        }
        override fun onServiceDisconnected(className: ComponentName?) {
            when(className?.className) {
                "br.com.xmrig.MiningService" -> {
                    miningService = null
                }
                "br.com.xmrig.services.XMRigAPIService" -> {
                    xmrigAPIService = null
                }
            }
        }
    };

    init {
        runBlocking(Dispatchers.IO) {
            arrayOf(
                    MiningService::class.java,
                    XMRigAPIService::class.java,
                    ThermalService::class.java
            ).onEach {
                launch(newSingleThreadContext("Thread-"+it.toString())) {
                    val intent = Intent(context, it)
                    context.bindService(intent, serverConnection, Context.BIND_AUTO_CREATE)
                    when(it) {
                        MiningService::class.java -> {
                            context.startForegroundService(intent)
                        }
                        else -> {
                            context.startService(intent)
                        }
                    }

                }
            }
        }
    }

    @Subscribe(threadMode = ThreadMode.ASYNC)
    fun onMessageEvent(event: StdoutEvent) {
        Log.d(this.name, "event name: " + event.javaClass.simpleName)
        val payload = Arguments.createMap()
        val strArr = arrayOf(event.value)
        payload.putArray("log", Arguments.fromArray(strArr))
        reactApplicationContext
                .getJSModule(RCTDeviceEventEmitter::class.java)
                .emit("onLog", payload)
    }

    @Subscribe(threadMode = ThreadMode.ASYNC)
    fun onMinerStartEvent(event: MinerStartEvent) {
        Log.d(this.name, "event name: " + event.javaClass.simpleName)
        this.isMining = true
        xmrigAPIService?.startSummaryUpdates()

        val payload = Arguments.createMap()
        payload.putBoolean("isWorking", true)
        reactApplicationContext
                .getJSModule(RCTDeviceEventEmitter::class.java)
                .emit("onStatusChange", payload)
    }

    @Subscribe(threadMode = ThreadMode.ASYNC)
    fun onMinerStopEvent(event: MinerStopEvent) {
        Log.d(this.name, "event name: " + event.javaClass.simpleName)
        this.isMining = false
        xmrigAPIService?.stopSummaryUpdates()

        val payload = Arguments.createMap()
        payload.putBoolean("isWorking", false)
        reactApplicationContext
                .getJSModule(RCTDeviceEventEmitter::class.java)
                    .emit("onStatusChange", payload)
    }

    @Subscribe(threadMode = ThreadMode.ASYNC)
    fun onPowerEvent(event: PowerEvent) {
        Log.d(this.name, "event name: " + event.javaClass.simpleName)
        val payload = Arguments.createMap()
        payload.putString("action", event.action.toString())
        if (event.value != null) {
            payload.putDouble("value", event.value!!.toDouble())
        }
        reactApplicationContext
                .getJSModule(RCTDeviceEventEmitter::class.java)
                .emit("onPower", payload)
    }

    @Subscribe(threadMode = ThreadMode.ASYNC)
    fun onMinerSummaryEvent(event: MinerSummaryEvent) {
        Log.d(this.name, "event name: " + event.javaClass.simpleName)
        if (event.value != null) {
            val payload = Arguments.createMap()
            payload.putString("data", event.value)

            reactApplicationContext
                    .getJSModule(RCTDeviceEventEmitter::class.java)
                    .emit("onSummary", payload)
        }
    }

    @Subscribe(threadMode = ThreadMode.ASYNC)
    fun onThermalEvent(event: ThermalEvent) {
        Log.d(this.name, "event name: " + event.javaClass.simpleName)
        val payload = Arguments.createMap()
        payload.putDouble("cpuTemperature", event.cpuTemperature.toDouble())

        reactApplicationContext
                .getJSModule(RCTDeviceEventEmitter::class.java)
                .emit("onThermal", payload)
    }

    @ReactMethod
    fun start(url:String, uuid:String) {
      
        try {
            miningService?.startMiner(url, uuid)
        } catch (e: RemoteException) {
            e.printStackTrace()
        }
    }

    @ReactMethod
    fun stop() {
        Log.d(this.name, "Stop has benn called from RN")
        try {
            miningService?.stopMiner()
            xmrigAPIService?.stopSummaryUpdates()
        } catch (e: RemoteException) {
            e.printStackTrace()
        }
        EventBus.getDefault().post(MinerStopEvent())
    }

    @ReactMethod
    fun availableProcessors(promise: Promise) {
        Log.d(this.name, "availableProcessors=" + Runtime.getRuntime().availableProcessors().toString())
        try {
            val availableProcessors = Integer.valueOf(Runtime.getRuntime().availableProcessors())
            promise.resolve(availableProcessors)
        } catch (e: Exception) {
            promise.reject("Runtime.getRuntime().availableProcessors()", e)
        }
    }

    @ReactMethod
    fun pauseMiner() {
        xmrigAPIService?.pauseMiner()
    }

    @ReactMethod
    fun resumeMiner() {
        xmrigAPIService?.resumeMiner()
    }

    override fun getName(): String {
        return "RNReactXMRigModule";
    }

    override fun initialize() {
        super.initialize()
        EventBus.getDefault().register(this)
    }

    override fun onCatalystInstanceDestroy() {
        super.onCatalystInstanceDestroy()
        EventBus.getDefault().unregister(this)
    }

    @ReactMethod
    fun addListener(eventName: String?) {
        when(eventName) {
            "onPower" -> {
                val bm = reactApplicationContext.getSystemService(BATTERY_SERVICE) as BatteryManager
                val batteryLevel = bm.getIntProperty(BatteryManager.BATTERY_PROPERTY_CAPACITY)

                val batteryStatus: Intent? = IntentFilter(Intent.ACTION_BATTERY_CHANGED).let { ifilter ->
                    reactApplicationContext.applicationContext.registerReceiver(null, ifilter)
                }

                val chargePlug: Int = batteryStatus?.getIntExtra(BatteryManager.EXTRA_PLUGGED, -1) ?: -1

                EventBus.getDefault().post(PowerEvent(PowerEventAction.BATTERY_CHANGED, batteryLevel))
                if (chargePlug > 0)   {
                    EventBus.getDefault().post(PowerEvent(PowerEventAction.POWER_CONNECTED))
                } else {
                    EventBus.getDefault().post(PowerEvent(PowerEventAction.POWER_DISCONNECTED))
                }
            }
        }

    }

    @ReactMethod
    fun removeListeners(count: Int?) {
        // Keep: Required for RN built in Event Emitter Calls.
    }


}